package ch.hslu.swk.g03.logger.component;

import java.io.FileInputStream;
import java.io.IOException;
import java.util.Properties;

import org.jacorb.orb.ORB;
import org.omg.CORBA.BooleanHolder;
import org.omg.CORBA.ORBPackage.InvalidName;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.CosNaming.NamingContextPackage.CannotProceed;
import org.omg.CosNaming.NamingContextPackage.NotFound;

import ch.hslu.corba.loggerinterface.CorbaLogLevel;
import ch.hslu.corba.loggerinterface.CorbaLoggerInterface;
import ch.hslu.corba.loggerinterface.CorbaLoggerInterfaceHelper;
import ch.hslu.corba.loggerinterface.CorbaLoggerSetupInterface;
import ch.hslu.corba.loggerinterface.CorbaLoggerInterfacePackage.CorbaLoggerCouldNotLogException;
import ch.hslu.loggerinterface.Logger;
import ch.hslu.loggerinterface.LoggerException;
import ch.hslu.loggerinterface.LoggerSetup;
import ch.hslu.loggerinterface.Loglevel;

public class LoggiCorba implements Logger {

	private ORB orb;
	private org.omg.CORBA.Object nameServiceObject;
	private org.omg.CORBA.Object clientObject;
	private NamingContextExt nameService;
	private CorbaLoggerInterface server;
	private CorbaLoggerSetupInterface loggerSetup;

	public LoggiCorba() {

		initializeCorba();
																																																			}

	/**
	 * Initializes the CORBA connection
	 */
	private void initializeCorba() {

		Properties prop = new Properties();

		System.setProperty("org.omg.CORBA.ORBClass", "org.jacorb.orb.ORB");
		System.setProperty("org.omg.CORBA.ORBSingletonClass", "org.jacorb.orb.ORBSingleton");

		try {
			
			prop.load(new FileInputStream("corba.properties"));
		} catch (IOException ex) {
			
			// Print the stack trace on error
			ex.printStackTrace();
		}

		// Initialize the orb
		orb = (ORB) ORB.init(new String[] {}, prop);

		// Resolve the naming service
		try {
			nameServiceObject = orb.resolve_initial_references("NameService");
			nameService = NamingContextExtHelper.narrow(nameServiceObject);

			// Resolve the corba server			
			clientObject = nameService.resolve(nameService.to_name(prop.getProperty("corba.bindname")));
			server = CorbaLoggerInterfaceHelper.narrow(clientObject);

		} catch (InvalidName | NotFound | CannotProceed | org.omg.CosNaming.NamingContextPackage.InvalidName ex) {
			
			// Print the stack trace on error
			ex.printStackTrace();
		}
	}

	/**
	 * Returns the current setup
	 */
	@Override
	public LoggerSetup getSetup() {

		loggerSetup = server.getCorbaLoggerSetup();
		LoggerSetup setup = new LoggiSetup();

		// Set the setup properties
		setup.setLoglevel(Loglevel.valueOf(loggerSetup.getCorbaLogLevel().toString()));
		setup.setSource(loggerSetup.getCorbaLogMessageSource().toString());

		return setup;
	}

	/**
	 * sets the setup
	 * 
	 * @param setup
	 *            the setup to set
	 */
	@Override
	public void setSetup(LoggerSetup arg0) {

		loggerSetup = server.getCorbaLoggerSetup();

		loggerSetup.setCorbaLogLevel(CorbaLogLevel.from_int(arg0.getLoglevel().getLoglevel()));
		loggerSetup.setCorbaLogMessageSource(arg0.getSource());

		server.setCorbaLoggerSetup(loggerSetup);
	}

	/**
	 * Sends log messages to a server. Logs a message with defined source and
	 * log level.
	 * 
	 * @param level
	 *            Sets the severity of the log message @see
	 *            ch.hslu.swk.f1301.loggerinterface.Loglevel
	 * @param message
	 *            Sets the message to log. Maximum of 10000 characters
	 */
	@Override
	public void log(Loglevel arg0, String arg1) throws LoggerException {

		try {
			BooleanHolder holder = new BooleanHolder();
			server.corbaLog(CorbaLogLevel.from_int(arg0.getLoglevel()), arg1, holder);

			if (!holder.value) {
				System.out.println(arg1 + " not logged");
			}
		} catch (CorbaLoggerCouldNotLogException ex) {
			
			// Print the stack trace on error
			ex.printStackTrace();
		}

	}

}
