package ch.hslu.swk.g03.logger.client;

import java.awt.Dimension;
import java.awt.EventQueue;
import javax.swing.JFrame;
import java.awt.BorderLayout;
import javax.swing.JPanel;
import java.awt.Color;
import javax.swing.DefaultComboBoxModel;
import javax.swing.JLabel;
import javax.swing.JButton;
import javax.swing.JOptionPane;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import java.awt.Font;
import javax.swing.JTextField;
import javax.swing.JComboBox;
import javax.swing.JTextPane;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.event.ListDataEvent;
import javax.swing.event.ListDataListener;
import ch.hslu.loggerinterface.Logger;
import ch.hslu.loggerinterface.LoggerFactory;
import ch.hslu.loggerinterface.LoggerSetup;
import ch.hslu.loggerinterface.Loglevel;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.io.IOException;
import java.net.URISyntaxException;
import java.util.Vector;

public class App implements ListDataListener {
	//SWING
	private JButton btnClose;
	private JButton btnSend;
	private JButton btnSave;
	private JComboBox<Loglevel> cboLogLvlMessage;
	private JComboBox<Loglevel> cboLogLevel;
	private JFrame frameLoggi;
	private JPanel pnlSetting;
	private JLabel lblLogLevel;
	private JLabel lblSource;
	private JLabel lblHost;
	private JLabel lblPort;
	private JPanel pnlTitle;
	private JLabel lblLoggi;
	private JPanel pnlBottom;
	private JPanel pnlMain;
	private JLabel lblMessage;
	private JScrollPane scrollPane;
	private JTabbedPane tabMain;
	private JTabbedPane sourceTabbedPane;
	private JTextField txtSource;
	private JTextField txtHost;
	private JTextField txtPort;
	private JTextPane tpMessage;

	//Listener
	private ChangeListener changeListener;
	
	//ComboBox Properties
	private DefaultComboBoxModel cboModel1;
	private DefaultComboBoxModel cboModel2;
	private Object selectedObject;
	private String selectedValue;
	private Vector<String> vLogLevel;
	private Vector<String> vLogLvlMessage;
	
	// Logger
	private Logger logger;
	private LoggerSetup loggerSetup;

	/**
	 * Create the application.
	 * @throws URISyntaxException 
	 * @throws IOException 
	 * @throws IllegalAccessException 
	 * @throws InstantiationException 
	 * @throws ClassNotFoundException 
	 */
	public App() throws ClassNotFoundException, InstantiationException, IllegalAccessException, IOException, URISyntaxException {
		//Initialize the GUI
		initialize();
		
		//Get the Logger from the factory.
		getLogger();
		
		//Get the Settings from the logger setup.
		getSettings();
	}
	
	/**
	 * Get the logger from the factory.
	 * @throws ClassNotFoundException
	 * @throws InstantiationException
	 * @throws IllegalAccessException
	 * @throws IOException
	 * @throws URISyntaxException
	 */
	private void getLogger() throws ClassNotFoundException, InstantiationException, IllegalAccessException, IOException, URISyntaxException {
		try {
			logger = LoggerFactory.getLogger();
			loggerSetup = logger.getSetup();
		} catch (Exception ex) {
			ex.getMessage();
			JOptionPane.showMessageDialog(frameLoggi, "Es konnte keine Verbindung zu Server hergestellt werden!\nLoggi wird nun beendet.", "Loggi", JOptionPane.ERROR_MESSAGE);
			System.exit(0);
		}
	}

	/**
	 * Get the Settings.
	 */
	private void getSettings() {
		try {
			txtSource.setText(loggerSetup.getSource());					//Source
			txtHost.setText(loggerSetup.getServer());					//Server
			txtPort.setText(String.valueOf(loggerSetup.getPort()));		//Port
			cboLogLevel.setSelectedItem(loggerSetup.getLoglevel());		//LogLevel
		} catch(Exception ex) {
			ex.getMessage();
			JOptionPane.showMessageDialog(frameLoggi, "Die Settings konnten nicht geladen werden!", "Loggi", JOptionPane.ERROR_MESSAGE);
		}
	}
	
	/**
	 * Get the LogLevel.
	 * @param cbo: JComboBox
	 * @return Selected LogLevel
	 */
	private Loglevel getLogLevel(JComboBox<Loglevel> cbo) {
		//Save the selected Item in a Object.
		selectedObject = cbo.getSelectedItem();
		
		//Convert the Object to a String.
		if (selectedObject != null) {
			selectedValue = selectedObject.toString();
		} else {
			selectedValue = "";
			throw new IllegalStateException("No Item selected!");
		}
		
		//Check the selected Value and return the LogLevel.
		switch(selectedValue) {
		case "OFF":
			return Loglevel.OFF;
		case "ERROR":
			return Loglevel.ERROR;
		case "WARNING":
			return Loglevel.WARNING;
		case "DEBUG":
			return Loglevel.DEBUG;
		case "INFO":
			return Loglevel.INFO;
		}
		return Loglevel.OFF;
	}

	/**
	 * Start application.
	 */
	public final void start() {
		EventQueue.invokeLater(new Runnable() {
			public void run() {
				try {
					frameLoggi.setVisible(true);
				} catch (Exception ex) {
					ex.getMessage();
				}
			}
		});
	}

	/**
	 * Initialize the contents of the frame.
	 */
	private void initialize() {
		//Frame
		frameLoggi = new JFrame();
		frameLoggi.setResizable(false);
		frameLoggi.setBounds(100, 100, 312, 303);
		frameLoggi.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
		frameLoggi.getContentPane().setLayout(new BorderLayout(0,0));
		
		//Top
		pnlTitle = new JPanel();
		pnlTitle.setBackground(new Color(245, 245, 245));
		pnlTitle.setPreferredSize(new Dimension(400,60));
		frameLoggi.getContentPane().add(pnlTitle, BorderLayout.NORTH);
		pnlTitle.setBounds(277, 13, 46, 14);
		pnlTitle.setLayout(null);
		lblLoggi = new JLabel("Loggi");
		lblLoggi.setFont(new Font("Tahoma", Font.PLAIN, 32));
		lblLoggi.setBounds(10, 10, 120, 38);
		pnlTitle.add(lblLoggi);
		
		//Bottom
		pnlBottom = new JPanel();
		pnlBottom.setBackground(new Color(245, 245, 245));
		pnlBottom.setPreferredSize(new Dimension(400,35));
		frameLoggi.getContentPane().add(pnlBottom, BorderLayout.SOUTH);
		pnlBottom.setLayout(null);
		btnClose = new JButton("close");
		btnClose.addActionListener(new ActionListener() {
			//Close the frame.
			public final void actionPerformed(ActionEvent arg0) {
				System.exit(0);
			}
		});
		btnClose.setBounds(204, 6, 89, 23);
		pnlBottom.add(btnClose);
		btnSend = new JButton("send");
		btnSend.addActionListener(new ActionListener() {
			public final void actionPerformed(ActionEvent arg0) {
				try {
					if (!tpMessage.getText().equals("")) {
						logger.log(getLogLevel(cboLogLvlMessage), tpMessage.getText());
						tpMessage.setText("");
					}
				} catch (Exception ex) {
					ex.getMessage();
					tpMessage.setForeground(Color.RED);
				}
			}
		});
		btnSend.setBounds(107, 6, 87, 23);
		pnlBottom.add(btnSend);
		
		//Tab main
		tabMain = new JTabbedPane(JTabbedPane.TOP);
		tabMain.setBackground(new Color(245, 245, 245));
		frameLoggi.getContentPane().add(tabMain, BorderLayout.CENTER);
		changeListener = new ChangeListener() {
			public final void stateChanged(ChangeEvent changeEvent) {
				sourceTabbedPane = (JTabbedPane) changeEvent.getSource();
		        if(sourceTabbedPane.getSelectedIndex() == 1) {
		        	btnSend.setEnabled(false);
		        } else {
		        	btnSend.setEnabled(true);
		        }
		      }
		};
		tabMain.addChangeListener(changeListener);
		pnlMain = new JPanel();
		pnlMain.setBackground(new Color(245, 245, 245));
		tabMain.add("Main", pnlMain);
		pnlMain.setLayout(null);
		lblMessage = new JLabel("Message:");
		lblMessage.setBounds(10, 11, 102, 14);
		pnlMain.add(lblMessage);
		tpMessage = new JTextPane();
		tpMessage.setBounds(10, 36, 281, 105);
		scrollPane = new JScrollPane(tpMessage);
		scrollPane.setBounds(10, 36, 281, 105);
		pnlMain.add(scrollPane);
		cboLogLvlMessage = new JComboBox<Loglevel>();
		cboLogLvlMessage.setBounds(75, 8, 216, 20);
		vLogLvlMessage = new Vector<String>();
		vLogLvlMessage.add("OFF");
		vLogLvlMessage.add("ERROR");
		vLogLvlMessage.add("WARNING");
		vLogLvlMessage.add("INFO");
		vLogLvlMessage.add("DEBUG");
		cboModel1 = new DefaultComboBoxModel(vLogLvlMessage);
		cboModel1.addListDataListener(this);
	    cboLogLvlMessage.setModel(cboModel1);
		pnlMain.add(cboLogLvlMessage);
		
		//Tab settings
		pnlSetting = new JPanel();
		pnlSetting.setBackground(new Color(245, 245, 245));
		tabMain.add("Setting", pnlSetting);
		pnlSetting.setLayout(null);
		lblLogLevel = new JLabel("Log-Level:");
		lblLogLevel.setBounds(10, 11, 67, 14);
		pnlSetting.add(lblLogLevel);
		lblSource = new JLabel("Source:");
		lblSource.setBounds(10, 36, 67, 14);
		pnlSetting.add(lblSource);
		lblHost = new JLabel("Host:");
		lblHost.setBounds(10, 61, 67, 14);
		pnlSetting.add(lblHost);
		txtSource = new JTextField();
		txtSource.setBounds(87, 33, 204, 20);
		pnlSetting.add(txtSource);
		txtSource.setColumns(10);
		txtHost = new JTextField();
		txtHost.setBounds(87, 58, 204, 20);
		pnlSetting.add(txtHost);
		txtHost.setColumns(10);
		cboLogLevel = new JComboBox<Loglevel>();
		cboLogLevel.setBounds(88, 8, 203, 20);
		vLogLevel = new Vector<String>();
		vLogLevel.add("OFF");
		vLogLevel.add("ERROR");
		vLogLevel.add("WARNING");
		vLogLevel.add("INFO");
		vLogLevel.add("DEBUG");
		cboModel2 = new DefaultComboBoxModel(vLogLevel);
		cboModel2.addListDataListener(this);
		cboLogLevel.setModel(cboModel2);
		pnlSetting.add(cboLogLevel);
		lblPort = new JLabel("Port:");
		lblPort.setBounds(10, 86, 67, 14);
		pnlSetting.add(lblPort);
		txtPort = new JTextField();
		txtPort.setColumns(10);
		txtPort.setBounds(87, 83, 204, 20);
		pnlSetting.add(txtPort);
		btnSave = new JButton("save");
		btnSave.addActionListener(new ActionListener() {
			public final void actionPerformed(ActionEvent arg0) {
				try {
					// Save LoggerSetup
					loggerSetup.setPort( Integer.parseInt(txtPort.getText()));
					loggerSetup.setServer(txtHost.getText());
					loggerSetup.setSource(txtSource.getText());
					loggerSetup.setLoglevel(getLogLevel(cboLogLevel));
					logger.setSetup(loggerSetup);
				} catch (Exception ex) {
					ex.getMessage();
					JOptionPane.showMessageDialog(frameLoggi, "Die Settings konnten nicht gespeichert werden!", "Loggi", JOptionPane.ERROR_MESSAGE);
				}
			}
		});
		btnSave.setBounds(204, 118, 87, 23);
		pnlSetting.add(btnSave);
	}

	/**
	 * contentsChanged.
	 */
	@Override
	public void contentsChanged(ListDataEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	/**
	 * intervalAdded.
	 */
	@Override
	public void intervalAdded(ListDataEvent arg0) {
		// TODO Auto-generated method stub
		
	}

	/**
	 * intervalRemoved.
	 */
	@Override
	public void intervalRemoved(ListDataEvent arg0) {
		// TODO Auto-generated method stub
		
	}
}
