package ch.hslu.swk.g03.logger.server;

import java.io.FileReader;
import java.util.Properties;

import ch.hslu.corba.loggerinterface.CorbaLoggerSetupInterface;
import ch.hslu.corba.loggerinterface.CorbaLoggerSetupInterfaceHelper;
import org.omg.CORBA.ORB;
import org.omg.CosNaming.NamingContextExt;
import org.omg.CosNaming.NamingContextExtHelper;
import org.omg.PortableServer.POA;

/**
 * LoggerServer Corba-Server.
 *
 * @author Felix Rohrer <felix.rohrer@stud.hslu.ch>
 */
public class ServerCorba {

    /**
     * Close the server.
     */
    private boolean close = false;


    /**
     * Create an initializes a new corba server.
     *
     * @param prop The properties to load.
     */
    public ServerCorba(final Properties prop) {

      System.setProperty("org.omg.CORBA.ORBClass", prop.getProperty("corba.orbclass"));
      System.setProperty("org.omg.CORBA.ORBSingletonClass", prop.getProperty("corba.orbsingletonclass"));

      Properties props = new Properties();
      try {
        props.load(new FileReader(prop.getProperty("corba.properties")));

        props.setProperty("ORBInitRef.NameService", prop.getProperty("corba.nameservice"));

        // Instantiieren des CORBA ORB
        ORB orb = ORB.init(new String[]{""}, props);

        //  Instantiieren des POA
        org.omg.CORBA.Object poaObject = orb.resolve_initial_references("RootPOA");
        POA poa = org.omg.PortableServer.POAHelper.narrow(poaObject);

        // Aktivieren des POA
        poa.the_POAManager().activate();

        // Kreiere neue Servants der entsprechenden Klassen
        ConcreteCorbaLoggerSetupInterfacePOA setup = new ConcreteCorbaLoggerSetupInterfacePOA();
        ConcreteCorbaLoggerInterfacePOA server = new ConcreteCorbaLoggerInterfacePOA();

        org.omg.CORBA.Object corbaServer = poa.servant_to_reference(server);
        org.omg.CORBA.Object corbaSetup  = poa.servant_to_reference(setup);

        // Instanziere den Namensdienst
        org.omg.CORBA.Object nso = orb.resolve_initial_references("NameService");
        NamingContextExt nameService = NamingContextExtHelper.narrow(nso);

        // Objektreferenz beim Namingservice anmelden
        // Rebind := Wiederverbinden (Verbindung ersetzen falls bereits bestehend)
        nameService.rebind(nameService.to_name(prop.getProperty("corba.bindname", "G03CorbaServer")), corbaServer);
        nameService.rebind(nameService.to_name(prop.getProperty("corba.bindnamesetup", "G03CorbaSetup")), corbaSetup);

        // get instance form ns and add to object instance of server
        org.omg.CORBA.Object objCorbaSetup = nameService.resolve(nameService.to_name(prop.getProperty("corba.bindnamesetup", "G03CorbaSetup")));
        CorbaLoggerSetupInterface objSetup = CorbaLoggerSetupInterfaceHelper.narrow(objCorbaSetup);
        
        server.setCorbaLoggerSetup(objSetup);
      }
      catch (Exception ex) {
	ex.printStackTrace();
        System.err.println("Unable to read jacorb properties in " + prop.getProperty("corba.properties"));
      }
    }

    /**
     * Closes the server.
     */
    public final void close() {
      this.close = true;
    }    
}
