/*
 * Copyright (c) 1995, 2008, Oracle and/or its affiliates. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

package ch.hslu.swk.g03.logger.viewer;

/*
 * SimpleTableDemo.java requires no other files.
 */


import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.SwingUtilities;

import java.awt.Dimension;
import java.awt.GridLayout;
/*
import java.awt.Point;
import javax.swing.JButton;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
*/

import javax.swing.table.DefaultTableModel;
import javax.swing.table.TableColumn;

import ch.hslu.swk.g03.logger.common.LogMessage;

import java.util.ArrayList;
import java.util.Enumeration;
import java.util.Properties;

public class Loggergui extends JPanel {

	private static final long serialVersionUID = 1L;
	static JTable table;
	static JScrollPane scrollPane;
	static JFrame frame;
	static String ScrollDirection;

	/**
	 * Default Constructor.
	 */
	public Loggergui() {
		super(new GridLayout(1, 0));

		final String[] columnNames = { "ID", "Timestamp", "Loglevel", "Message",
				"Source", "Computer" };

		final Object[][] data = { };

		table = new JTable(new DefaultTableModel(data, columnNames));
		table.setEnabled(Boolean.FALSE);
		table.setPreferredScrollableViewportSize(new Dimension(500, 300));
		table.setFillsViewportHeight(true);

		Enumeration<TableColumn> en = table.getColumnModel().getColumns();
        while (en.hasMoreElements()) {
            TableColumn tc = en.nextElement();
            tc.setCellRenderer(new CustomCellRenderer());
        }
		
		// Create the scroll pane and add the table to it.
		scrollPane = new JScrollPane(table);

		// Add the scroll pane to this panel.
		add(scrollPane);
		
		final Properties mySettings = new Properties();
		
		try {
			mySettings.load(FetchThread.class.getClassLoader()
					.getResourceAsStream("viewer.properties"));
			ScrollDirection = mySettings.getProperty("ScrollDirection");
		}catch(Exception ex){
			ScrollDirection = "desc";
		}
	}

	/**
	 * Add some Messages to GUI.
	 * @param messages ArrayList of LogMessage
	 */
	public static void addMessage(final ArrayList<LogMessage> messages) {
		for (final LogMessage logMessage : messages) {
			addMessage(logMessage);
		}
	}

	/**
	 * Add single Message to GUI.
	 * @param mess LogMessage
	 */
	public static void addMessage(final LogMessage mess) {
		
		SwingUtilities.invokeLater(new Runnable() {
		    public void run() {
		      // Here, we can safely update the GUI
		      // because we'll be called from the
		      // event dispatch thread

		    	final DefaultTableModel model = (DefaultTableModel) table.getModel();
		    	
		    	if(ScrollDirection=="desc"){
		    		model.insertRow(0, new Object[] { mess.getId(), mess.getTimestamp(),
						mess.getLoglevel(), mess.getMessage(), mess.getSource(),
						mess.getComputer() });
		    	}else{
		    		model.addRow(new Object[] { mess.getId(), mess.getTimestamp(),
							mess.getLoglevel(), mess.getMessage(), mess.getSource(),
							mess.getComputer() });
		    		int last = table.getModel().getRowCount() - 1;
		    		table.scrollRectToVisible(table.getCellRect(last, 0, true));
		    	}
		    	
		    	Enumeration<TableColumn> en = table.getColumnModel().getColumns();
		        while (en.hasMoreElements()) {
		            TableColumn tc = en.nextElement();
		            tc.setCellRenderer(new CustomCellRenderer());
		        }
		    	
				table.revalidate();
				
		    }
		  });
		
	}

	/**
	 * Create the GUI and show it. For thread safety, this method should be
	 * invoked from the event-dispatching thread.
	 */
	public static void createAndShowGUI() {
		// Create and set up the window.
		frame = new JFrame("Logger Viewer");
		frame.setLocation(100, 100);
		frame.setSize(500, 300);
		frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);

		// Create and set up the content pane.
		final Loggergui newContentPane = new Loggergui();
		newContentPane.setOpaque(true); // content panes must be opaque
		frame.setContentPane(newContentPane);

		// Display the window.
		frame.pack();
		frame.setVisible(true);
	}

	/**
	 * main method
	 */
	public static void main(final String[] args) {
		// Schedule a job for the event-dispatching thread:
		// creating and showing this application's GUI.
		javax.swing.SwingUtilities.invokeLater(new Runnable() {
			@Override
			public void run() {
				createAndShowGUI();
				try {
					final FetchThread myfetchThread = FetchThread.getInstance();
					myfetchThread.start();
				} catch (Exception ex) {
					System.out.println("rmi error " + ex.getMessage());
				}
			}
		});
	}
}